using System;
using System.IO;
using System.Text;
using System.Text.RegularExpressions;
using ICSharpCode.SharpZipLib.Zip;

namespace CmdAceUpdater
{
	public class FileSystem
	{
		public static string GetAddonDirectory(string Addon)
		{
			return Path.Combine(Settings.AddonsDir, Addon);
		}

		public static VersionNumber GetAddonRevision(string Addon)
		{
			try
			{
				string AddonDir = GetAddonDirectory(Addon);
				VersionNumber Best = VersionNumber.NO_VERSION;
				if (Directory.Exists(AddonDir))
				{
					string[] FilePaths = Directory.GetFiles(AddonDir, "Changelog*.txt");
					foreach (string FilePath in FilePaths)
					{
						string FileName = Path.GetFileName(FilePath);
						Match m = Regex.Match(FileName, "(?<=r)[\\d\\.]+\\d");
						if (!m.Success) continue;
						VersionNumber Val = VersionNumber.Parse(m.ToString());
						if (Val > Best) Best = Val;
					}
				}
				return Best;
			}
			catch
			{
				return VersionNumber.NO_VERSION;
			}
		}

		public static void UnzipAll(string ZipFilePath, string DestinationFolder)
		{
			FastZip Zip = new FastZip();
			Zip.ExtractZip(ZipFilePath, DestinationFolder, "");
		}

		public static bool IsSvnWorkingCopy(string AddonDir)
		{
			if (Directory.Exists(AddonDir))
			{
				string[] SvnDirs = Directory.GetDirectories(AddonDir, ".svn", SearchOption.TopDirectoryOnly);
				return SvnDirs.Length > 0;
			}
			else return false;
		}

		public static bool IsPackage(string AddonDir)
		{
			return File.Exists(Path.Combine(AddonDir, "filelist.wau"));
		}

		public static void HandlePackage(string AddonDir, bool Pack)
		{
			string Addon = Path.GetFileName(AddonDir);
			StreamReader SubAddonList = new StreamReader(Path.Combine(AddonDir, "filelist.wau"), Encoding.Default);
			while (SubAddonList.Peek() >= 0)
			{
				string Line = SubAddonList.ReadLine();
				if (Line.Length > 0)
				{
					string SubAddon = Line, NewDir, SubDir;
					if (Line.StartsWith("@"))
					{
						Line = Line.Substring(1);
						SubAddon = String.Format("{0}_{1}", Addon, Line);
					}
					NewDir = Path.Combine(Directory.GetParent(AddonDir).ToString(), SubAddon);
					SubDir = Path.Combine(AddonDir, Line);
					try
					{
						if (Pack)
						{
							try { Directory.Delete(SubDir, true); } catch (IOException) {}
							Directory.Move(NewDir, SubDir);
						}
						else
						{
							try { Directory.Delete(NewDir, true); } catch (IOException) {}
							Directory.Move(SubDir, NewDir);
						}
					}
					catch (IOException) {}
				}
			}
			SubAddonList.Close();
		}

		public static void Remove()
		{
			if (File.Exists(Settings.RemoveFileName) && Settings.Remove)
			{
				StreamReader RemoveList = new StreamReader(Settings.RemoveFileName, Encoding.Default);
				while (RemoveList.Peek() >= 0)
				{
					string Line = RemoveList.ReadLine();
					if (Line.Length > 0)
					{
						try { Directory.Delete(Path.Combine(Settings.AddonsDir, Line), true); }
						catch (IOException) { }
						try { File.Delete(Path.Combine(Settings.AddonsDir, Line)); }
						catch (IOException) { }
					}
				}
				RemoveList.Close();
			}
		}
	}
}
