/*
CmdAceUpdater
by lys, stipek@gmail.com
released under the GPLv3
*/

using System;
using System.IO;

namespace CmdAceUpdater
{
	static class CmdAceUpdater
	{
		private static void InvalidSwitch(string Switch)
		{
			Output.Error(String.Format("invalid usage of the \"{0}\" switch", Switch));
		}

		private static void Main(string[] Args)
		{
			Console.WriteLine("CmdAceUpdater v1.0b9");
			bool DoNotWriteSettings = false;
			Settings.ReadConfigFile();
			Output.Init();
			if (Args.Length == 0) Output.Info(String.Format("use the \"-h\" switch to display the usage info"));
			else
			{
				for (int i = 0; i < Args.Length; i++)
				{
					if (Args[i] == "-h")
					{
						Console.WriteLine(" -resetsettings\t\tresets the settings to default values");
						Console.WriteLine(" -c [setting] [value]\tdisplays or modifies the settings");
						Console.WriteLine(" -i [addon]\t\tinstalls the specified addon");
						Console.WriteLine(" -r [addon]\t\tremoves the specified addon");
						Console.WriteLine(" -u\t\t\tupdates all installed addons");
					}
					else if (Args[i] == "-resetsettings")
					{
						try { File.Delete(Path.Combine(Settings.ProfileDir, Settings.ConfigFileName)); }
						catch (IOException) { }
						Output.Info("the settings have been reset");
						DoNotWriteSettings = true;
					}
					else if (Args[i] == "-c")
					{
						if (i + 2 < Args.Length)
						{
							try
							{
								if (Args[i + 1] == "WowDir") Settings.WowDir = Args[i + 2];
								else if (Args[i + 1] == "AceRepositoryUrl") Settings.AceRepositoryUrl = Args[i + 2];
								else if (Args[i + 1] == "SkipExternals") Settings.SkipExternals = bool.Parse(Args[i + 2]);
								else if (Args[i + 1] == "SkipVersionCheck") Settings.SkipVersionCheck = bool.Parse(Args[i + 2]);
								else if (Args[i + 1] == "DeleteBeforeExtracting") Settings.DeleteBeforeExtracting = bool.Parse(Args[i + 2]);
								else if (Args[i + 1] == "Logging") Settings.Logging = bool.Parse(Args[i + 2]);
								else if (Args[i + 1] == "SkipSvnWorkingCopies") Settings.SkipSvnWorkingCopies = bool.Parse(Args[i + 2]);
								else if (Args[i + 1] == "HandlePackages") Settings.HandlePackages = bool.Parse(Args[i + 2]);
								else if (Args[i + 1] == "Remove") Settings.Remove = bool.Parse(Args[i + 2]);
								else Output.Error(String.Format("\"{0}\" is not a valid setting name", Args[i + 1]));
							}
							catch (System.FormatException) { Output.Error(String.Format("\"{0}\" is not a valid value for the \"{1}\" setting", Args[i + 2], Args[i + 1])); }
							i += 2;
						}
						else
						{
							Console.WriteLine("WowDir=" + Settings.WowDir);
							Console.WriteLine("AceRepositoryUrl=" + Settings.AceRepositoryUrl);
							Console.WriteLine("SkipExternals=" + Settings.SkipExternals.ToString());
							Console.WriteLine("SkipVersionCheck=" + Settings.SkipVersionCheck.ToString());
							Console.WriteLine("DeleteBeforeExtracting=" + Settings.DeleteBeforeExtracting.ToString());
							Console.WriteLine("Logging=" + Settings.Logging.ToString());
							Console.WriteLine("SkipSvnWorkingCopies=" + Settings.SkipSvnWorkingCopies.ToString());
							Console.WriteLine("HandlePackages=" + Settings.HandlePackages.ToString());
							Console.WriteLine("Remove=" + Settings.Remove.ToString());
						}
					}
					else if (Args[i] == "-i")
					{
						if (i + 1 < Args.Length)
						{
							bool Old = Settings.SkipVersionCheck;
							Settings.SkipVersionCheck = true;
							Worker.Install(Args[i + 1]);
							Settings.SkipVersionCheck = Old;
							i++;
						}
						else InvalidSwitch("-i");
					}
					else if (Args[i] == "-r")
					{
						if (i + 1 < Args.Length)
						{
							Worker.Uninstall(Args[i + 1], false);
							i++;
						}
						else InvalidSwitch("-r");
					}
					else if (Args[i] == "-u")
					{
						Output.Info("starting the ace addons update");
						int Count = Worker.Update();
						Output.Info("update finished - addons updated: " + Count.ToString());
					}
					else Output.Error(String.Format("\"{0}\" is not a valid switch", Args[i]));
				}
			}
			FileSystem.Remove();
			if (!DoNotWriteSettings) Settings.WriteConfigFile();
		}
	}
}
