using System;
using System.IO;
using System.Net;
using System.Xml;
using System.Collections.Generic;
using ICSharpCode.SharpZipLib.GZip;

namespace CmdAceUpdater
{
	internal static class Server
	{
		private const string IndexFileNameExt = "latest.xml";
		private const string IndexFileNameNoExt = "latest-noext.xml";
		private static XmlDocument Index = new XmlDocument();
		private static SortedDictionary<string, XmlNode> _FilesLookup;

		private static WebClient GetWebConfiguredWebClient()
		{
			WebClient Result = new WebClient();
			Result.Headers.Add("User-Agent", String.Format("{0}/{1}", "CmdAceUpdater", 1));
			return Result;
		}

		public static SortedDictionary<string, XmlNode> FilesLookup
		{
			get
			{
				if (_FilesLookup == null)
				{
					XmlDocument x = Index;
					XmlNodeList n = x.GetElementsByTagName("item");
					_FilesLookup = new SortedDictionary<string, XmlNode>();
					foreach (XmlNode addon in n)
					{
						foreach (XmlNode node in addon.ChildNodes)
							if (node.Name == "title")
							{
								_FilesLookup.Add(node.FirstChild.Value, addon);
								break;
							}
					}
				}
				return Server._FilesLookup;
			}
		}

		public static void GetIndexFile()
		{
			string Address, FileName = Path.Combine(Settings.ProfileDir, "index.xml");
			if (Settings.SkipExternals) Address = String.Format("{0}/{1}", Settings.AceRepositoryUrl, IndexFileNameNoExt);
			else Address = String.Format("{0}/{1}", Settings.AceRepositoryUrl, IndexFileNameExt);
			using (WebClient wc = GetWebConfiguredWebClient())
			{
				try
				{
					wc.Headers.Add("Accept-Encoding:  gzip, deflate");
					wc.DownloadFile(Address, FileName);
					FileStream fs = new FileStream(FileName, FileMode.Open);
					GZipInputStream gzi = new GZipInputStream(fs);
					Index.Load(gzi);
				}
				catch (System.Net.WebException)
				{
					Output.Error("unable to download the index file");
				}
			}
		}

		public static VersionNumber GetAddonRevision(string Addon)
		{
			try
			{
				XmlNode XmlAddon = FilesLookup[Addon];
				foreach (XmlNode Node in XmlAddon.ChildNodes)
					if (Node.Name == "wowaddon:version")
						return VersionNumber.Parse(Node.FirstChild.Value);
				return VersionNumber.NO_VERSION;
			}
			catch
			{
				return VersionNumber.NO_VERSION;
			}
		}

		public static void DownloadAddon(string Addon, string LocalZipPath)
		{
			DownloadFile(GetZipFileUrl(Addon), LocalZipPath);
		}

		private static void DownloadFile(string FileUrl, string LocalFileName)
		{
			using (WebClient wc = GetWebConfiguredWebClient())
				wc.DownloadFile(FileUrl, LocalFileName);
		}

		public static string GetZipFileName(string Addon, VersionNumber Rev)
		{
			return String.Format("{0}-r{1}.zip", Addon, Rev);
		}

		public static string GetZipFileUrl(string Addon)
		{
			string Url = null;
			XmlNode XmlAddon = FilesLookup[Addon];
			foreach (XmlNode Node in XmlAddon.ChildNodes)
				if (Node.Name == "enclosure")
					Url = Node.Attributes.GetNamedItem("url").Value;
			return Url;
		}
	}
}
