using System;
using System.IO;
using System.Text;
using System.Windows.Forms;
using Microsoft.Win32;

namespace CmdAceUpdater
{
	public class Settings
	{
		public static string ProfileDir, ConfigFileName, RemoveFileName, LogFileName, WowDir = @"C:\Program Files\World of Warcraft", AddonsDir = @"C:\Program Files\World of Warcraft\Interface\Addons", AceRepositoryUrl = "http://files.wowace.com/";
		public static bool SkipExternals = false, SkipVersionCheck = false, DeleteBeforeExtracting = false, OutputInfo = true, OutputError = true, OutputRaw = true, Logging = false, SkipSvnWorkingCopies = true, HandlePackages = true, Remove = true;

		private static string GetInstallFolder()
		{
			RegistryKey Key = Registry.LocalMachine.OpenSubKey(@"SOFTWARE\Blizzard Entertainment\World of Warcraft", false);
			if (Key != null) return Key.GetValue("InstallPath", "") as string;
			return null;
		}

		public static void ReadConfigFile()
		{
			ProfileDir = Path.Combine(Path.GetDirectoryName(Application.ExecutablePath), "conf");
			ConfigFileName = Path.Combine(ProfileDir, "CmdAceUpdater.conf");
			RemoveFileName = Path.Combine(ProfileDir, "Remove.conf");
			LogFileName = Path.Combine(ProfileDir, "CmdAceUpdater.log");
			WowDir = GetInstallFolder();
			if (!Directory.Exists(ProfileDir)) Directory.CreateDirectory(ProfileDir);
			if (File.Exists(ConfigFileName))
			{
				StreamReader Conf = new StreamReader(ConfigFileName, Encoding.Default);
				while (Conf.Peek() >= 0)
				{
					string Line = Conf.ReadLine();
					if (Line.Length > 0)
					{
						string[] Setting = Line.Split('=');
						if (Setting[0] == "WowDir") WowDir = Setting[1];
						else if (Setting[0] == "AceRepositoryUrl") AceRepositoryUrl = Setting[1];
						else if (Setting[0] == "SkipExternals") SkipExternals = bool.Parse(Setting[1]);
						else if (Setting[0] == "SkipVersionCheck") SkipVersionCheck = bool.Parse(Setting[1]);
						else if (Setting[0] == "DeleteBeforeExtracting") DeleteBeforeExtracting = bool.Parse(Setting[1]);
						else if (Setting[0] == "Logging") Logging = bool.Parse(Setting[1]);
						else if (Setting[0] == "SkipSvnWorkingCopies") SkipSvnWorkingCopies = bool.Parse(Setting[1]);
						else if (Setting[0] == "HandlePackages") HandlePackages = bool.Parse(Setting[1]);
						else if (Setting[0] == "Remove") Remove = bool.Parse(Setting[1]);
					}
				}
				Conf.Close();
				AddonsDir = Path.Combine(Path.Combine(WowDir, "Interface"), "Addons");
			}
		}

		public static void WriteConfigFile()
		{
			StreamWriter Conf = new StreamWriter(ConfigFileName, false, Encoding.Default);
			Conf.WriteLine("WowDir=" + WowDir);
			Conf.WriteLine("AceRepositoryUrl=" + AceRepositoryUrl);
			Conf.WriteLine("SkipExternals=" + SkipExternals);
			Conf.WriteLine("SkipVersionCheck=" + SkipVersionCheck);
			Conf.WriteLine("DeleteBeforeExtracting=" + DeleteBeforeExtracting);
			Conf.WriteLine("Logging=" + Logging);
			Conf.WriteLine("SkipSvnWorkingCopies=" + SkipSvnWorkingCopies);
			Conf.WriteLine("HandlePackages=" + HandlePackages);
			Conf.WriteLine("Remove=" + Remove);
			Conf.Close();
		}
	}
}
