using System;

namespace CmdAceUpdater
{
	public class VersionNumber
	{
		private int _Major = 0;

		public int Major
		{
			get { return _Major; }
		}
		private int _Minor = 0;

		public int Minor
		{
			get { return _Minor; }
		}

		public static readonly VersionNumber NO_VERSION = new VersionNumber(0, 0);

		public VersionNumber(int Major, int Minor)
		{
			_Major = Major;
			_Minor = Minor;
		}
		public VersionNumber(int Major)
		{
			_Major = Major;
		}

		private enum compare { LESSER, GREATER, EQUAL };

		private compare Compare(VersionNumber with)
		{
			compare c;

			if (_Major > with._Major)
				c = compare.GREATER;
			else if (_Major < with._Major)
				c = compare.LESSER;
			else
			{
				if (_Minor > with._Minor)
					c = compare.GREATER;
				else if (_Minor < with._Minor)
					c = compare.LESSER;
				else
					c = compare.EQUAL;
			}

			return c;
		}

		public override bool Equals(object o)
		{
			if (o is VersionNumber)
				return Compare((VersionNumber)o) == compare.EQUAL;

			return false;
		}

		public override int GetHashCode()
		{
			return _Major << 16 + _Minor;
		}

		public override string ToString()
		{
			if (_Minor > 0)
				return String.Format("{0}.{1}", _Major, _Minor);

			return String.Format("{0}", _Major);
		}

		public static bool operator <(VersionNumber a, VersionNumber b)
		{
			return a.Compare(b) == compare.LESSER;
		}
		public static bool operator >(VersionNumber a, VersionNumber b)
		{
			return a.Compare(b) == compare.GREATER;
		}
		public static bool operator ==(VersionNumber a, VersionNumber b)
		{
			return a.Compare(b) == compare.EQUAL;
		}
		public static bool operator !=(VersionNumber a, VersionNumber b)
		{
			return !(a.Compare(b) == compare.EQUAL);
		}

		public static VersionNumber Parse(string s)
		{
			int min = 0, maj = 0;
			char[] d = { '.' };

			string[] ss = s.Split(d);

			if (ss.GetLength(0) > 0)
			{
				int i = ss.GetLowerBound(0);
				maj = Int32.Parse(ss[i]);
				if (ss.GetLength(0) > 1)
					min = Int32.Parse(ss[++i]);
			}

			return new VersionNumber(maj, min);
		}
	}
}
