using System;
using System.IO;
using System.Collections.Generic;

namespace CmdAceUpdater
{
	public class Worker
	{
		private static List<string> InstalledAceAddons = new List<string>();
		private static bool Ready = false;

		public static void InstallAddon(string Addon)
		{
			string AddonZipFile = Path.Combine(Settings.ProfileDir, "temp.zip"), AddonDir = Path.Combine(Settings.AddonsDir, Addon);
			try
			{
				Server.DownloadAddon(Addon, AddonZipFile);
			}
			catch (System.Net.WebException)
			{
				Output.Error("unable to download the addon zip file");
			}
			if (Settings.DeleteBeforeExtracting) Uninstall(Addon, true);
			else if (FileSystem.IsPackage(AddonDir) && Settings.HandlePackages) FileSystem.HandlePackage(AddonDir, true);
			FileSystem.UnzipAll(AddonZipFile, Settings.AddonsDir);
			if (FileSystem.IsPackage(AddonDir) && Settings.HandlePackages) FileSystem.HandlePackage(AddonDir, false);
			File.Delete(AddonZipFile);
		}

		public static bool Install(string Addon)
		{
			if (!Ready) { GetAddonList(); Ready = true; }
			if (Settings.SkipSvnWorkingCopies && FileSystem.IsSvnWorkingCopy(Path.Combine(Settings.AddonsDir, Addon))) return false;
			VersionNumber RemoteRev = Server.GetAddonRevision(Addon);
			VersionNumber LocalRev = FileSystem.GetAddonRevision(Addon);
			if (Settings.SkipVersionCheck)
			{
				Output.Raw("u: " + Addon + " -> " + RemoteRev.ToString() + "\n");
				InstallAddon(Addon);
				return true;
			}
			else if (RemoteRev > LocalRev)
			{
				Output.Raw("u: " + Addon + " -> " + RemoteRev.ToString() + "\n");
				InstallAddon(Addon);
				return true;
			}
			else
			{
				Output.Raw("i: " + Addon + " = " + RemoteRev.ToString() + "\n");
				return false;
			}
		}

		public static int Update()
		{
			if (!Ready) { GetAddonList(); Ready = true; }
			int Count = 0;
			foreach (string Addon in InstalledAceAddons) if (Install(Addon)) Count++;
			return Count;
		}

		public static void Uninstall(string Addon, bool Silent)
		{
			try
			{
				VersionNumber LocalRev = FileSystem.GetAddonRevision(Addon);
				string AddonDir = Path.Combine(Settings.AddonsDir, Addon);
				if (FileSystem.IsPackage(AddonDir) && Settings.HandlePackages) FileSystem.HandlePackage(AddonDir, true);
				Directory.Delete(AddonDir, true);
				if (!Silent) Output.Raw("r: " + Addon + " = " + LocalRev.ToString() + "\n");
			}
			catch (IOException) { if (!Silent) Output.Error(String.Format("addon \"{0}\" is not installed", Addon)); }
		}

		public static void GetAddonList()
		{
			Server.GetIndexFile();
			List<string> InstalledAddons = new List<string>();
			InstalledAddons.AddRange(Directory.GetDirectories(Settings.AddonsDir));
            for (int i = 0; i < InstalledAddons.Count; i++) InstalledAddons[i] = Path.GetFileName(InstalledAddons[i]);
			foreach (string Addon in InstalledAddons)
				if (Server.GetAddonRevision(Addon) != VersionNumber.NO_VERSION)
					InstalledAceAddons.Add(Addon);
		}
	}
}
